/** Rill v0.0.557 https://www.npmjs.com/package/rill License: MIT*/
var GeneratorFunction, Request, Response, Rill, co, compose, expect, http, pathToReg, respond,
  slice = [].slice;

co = require("co");

pathToReg = require("path-to-regexp");

compose = require("./util/compose");

expect = require("./util/expect");

Request = require("./request");

Response = require("./response");

respond = require("./util/respond");

http = (require("http"));

GeneratorFunction = (function*() {
  return;
}).constructor;

Rill = (function() {

  /*
  	 * Creates an isomorphic Rill application that can listen for browser and server page requests.
  	#
  	 * @constructor
   */
  var j, len, method, ref;

  function Rill() {
    if (!(this instanceof Rill)) {
      return new Rill();
    }
    this.middleware = [];
  }


  /*
  	 * Starts the server or browser hijacker.
   */

  Rill.prototype.listen = function() {
    var fn, ref;
    fn = co.wrap(compose([respond].concat(this.middleware)));
    return (ref = http.createServer(function(req, res) {
      if (res == null) {
        res = {};
      }
      res.statusCode = 404;
      req = new Request(req);
      res = new Response(res);
      return fn.call({}, req, res).then(function() {
        return {
          redirected: Boolean(req.redirectURL)
        };
      })["catch"](function(err) {
        throw err;
      });
    })).listen.apply(ref, arguments);
  };


  /*
  	 * Simple syntactic sugar for functions that wish to modify the current application instance.
  	#
  	 * @param {Function} transformer
   */

  Rill.prototype.setup = function(transformer) {
    expect(transformer(this) === this, "Setup failed, transformer function must return the modified application.");
    return this;
  };


  /*
  	 * Attaches a seperate rill application at a given path.
  	 * req.path and req.params will be updated to remove the provided path.
  	#
  	 * @param {Rill} app
   */

  Rill.prototype.mount = function(path, app) {
    var handler;
    if (typeof path !== "string") {
      app = path;
      path = "/";
    }
    if (path === "/") {
      this.middleware = this.middleware.concat(app.middleware);
      return this;
    }
    expect(path[0] === "/", "Mount path must begin with '/'.");
    expect(app.middleware, "Can only mount Rill applications.");
    handler = compose(app.middleware);
    this.middleware.push(function*(req, res, next) {
      var newPath, prevPath;
      if (req.path.indexOf(path) !== 0) {
        return (yield* next);
      }
      prevPath = req.path;
      newPath = req.path = req.path.replace(path, "");
      (yield* handler.call(this, req, res, (function*() {
        req.path = prevPath;
        (yield* next);
        return req.path = newPath;
      })()));
      return req.path = prevPath;
    });
    return this;
  };


  /*
  	 * Create functions for each http verb that will accept a path url and a middleware function to use.
  	 * A "use" method is also attached that will not check the request method.
  	#
  	 * @param {String} path
  	 * @param {Generator...} handlers
   */

  ref = ["USE", "OPTIONS", "HEAD", "GET", "PUT", "POST", "DELETE"];
  for (j = 0, len = ref.length; j < len; j++) {
    method = ref[j];
    Rill.prototype[method.toLowerCase()] = (function(method) {
      return function() {
        var handler, handlers, k, keys, len1, path;
        path = arguments[0], handlers = 2 <= arguments.length ? slice.call(arguments, 1) : [];
        if (typeof path !== "string") {
          handlers = [path].concat(handlers);
          path = "/";
        }
        for (k = 0, len1 = handlers.length; k < len1; k++) {
          handler = handlers[k];
          expect(handler instanceof GeneratorFunction, "Can only use GeneratorFunctions.");
        }
        handlers = compose(handlers);
        if (path === "/" && method === "USE") {
          this.middleware = this.middleware.concat(handlers);
          return this;
        }
        expect(path[0] === "/", "At path must begin with '/'.");
        path = pathToReg(path, keys = []);
        this.middleware.push(function*(req, res, next) {
          var i, l, len2, match, name, optional, ref1;
          if (!((req.method === method || method === "USE") && (match = req.path.match(path)))) {
            return (yield* next);
          }
          if (req.params == null) {
            req.params = {};
          }
          for (i = l = 0, len2 = keys.length; l < len2; i = ++l) {
            ref1 = keys[i], name = ref1.name, optional = ref1.optional;
            if (!(((req.params[name] = match[i + 1]) != null) || optional)) {
              return (yield* next);
            }
          }
          return (yield* handler.call(this, req, res, next));
        });
        return this;
      };
    })(method);
  }

  return Rill;

})();

module.exports = Rill;
